/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import lombok.AllArgsConstructor;
import nl.tudelft.labracore.api.dto.*;
import nl.tudelft.tam.service.CourseService;

@Controller
@AllArgsConstructor
@RequestMapping("course")
public class CourseController {

	private CourseService courseService;

	/**
	 * Get the overview for a specific course.
	 *
	 * @param  id    The id of the course to retrieve.
	 * @param  model The model to fill out for Thymeleaf template resolution.
	 * @return       The thymeleaf template to resolve.
	 */
	@GetMapping("{id}")
	@PreAuthorize("@authorisationService.canManageCourse(#id)")
	public String viewCourse(@PathVariable Long id, Model model) {
		CourseDetailsDTO course = courseService.getCourseById(id);
		model.addAttribute("course", course);
		return "course/view";
	}

}
