/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import nl.tudelft.tam.dto.create.TrainingTypeCreateDTO;
import nl.tudelft.tam.dto.patch.TrainingTypePatchDTO;
import nl.tudelft.tam.service.TrainingTypeService;

@Controller
@RequestMapping("training-type")
public class TrainingTypeController {

	@Autowired
	private TrainingTypeService trainingTypeService;

	/**
	 * Create a new training type.
	 *
	 * @param  programId The program id that the training type belongs to.
	 * @param  name      The name of the training type
	 * @return           The redirect to the training page.
	 */
	@PostMapping
	@PreAuthorize("@authorisationService.isCoordinatorForProgram(#createDTO.programId)")
	public String createTrainingType(TrainingTypeCreateDTO createDTO) {
		trainingTypeService.createNewTrainingType(createDTO);
		return "redirect:/coordinator/training/" + createDTO.getProgramId();
	}

	/**
	 * Update a training type.
	 *
	 * @param  programId      The program id that the training type belongs to.
	 * @param  trainingTypeId The id of the training type
	 * @param  name           The new name of the training type
	 * @return                The redirect to the training page.
	 */
	@PatchMapping("update/{programId}/{trainingTypeId}")
	@PreAuthorize("@authorisationService.isCoordinatorForProgram(#programId)")
	public String updateTrainingType(@PathVariable Long programId,
			@PathVariable Long trainingTypeId,
			TrainingTypePatchDTO patchDTO) {
		trainingTypeService.updateTrainingType(patchDTO, trainingTypeId);
		return "redirect:/coordinator/training/" + programId;
	}

	/**
	 * Delete a training type.
	 *
	 * @param  programId      The program id that the training type belongs to.
	 * @param  trainingTypeId The id of the training type
	 * @return                The redirect to the training page.
	 */
	@DeleteMapping("delete/{programId}/{trainingTypeId}")
	@PreAuthorize("@authorisationService.isCoordinatorForProgram(#programId)")
	public String deleteTrainingType(@PathVariable Long programId,
			@PathVariable Long trainingTypeId) {
		trainingTypeService.deleteTrainingType(trainingTypeId);

		return "redirect:/coordinator/training/" + programId;
	}
}
