/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.create;

import java.time.LocalDate;

import org.springframework.format.annotation.DateTimeFormat;

import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import lombok.*;
import nl.tudelft.librador.dto.create.Create;
import nl.tudelft.tam.enums.OfferType;
import nl.tudelft.tam.model.JobOffer;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class JobOfferCreateDTO extends Create<JobOffer> {

	@NotNull
	private Long editionId;

	private String name;

	private String contractName;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate contractStartDate;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate contractEndDate;

	private String baanCode;

	@Positive
	private Integer maxHours;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate deadline;

	private String description;

	private String hiringMessage;

	private String rejectMessage;

	@Builder.Default
	private Boolean requireApplicationText = false;

	@Builder.Default
	private Boolean draft = false;

	@Builder.Default
	private OfferType offerType = OfferType.TA_OR_HEAD_TA;

	@Override
	public Class<JobOffer> clazz() {
		return JobOffer.class;
	}
}
