/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.patch;

import org.modelmapper.ModelMapper;

import jakarta.persistence.Embedded;
import lombok.*;
import nl.tudelft.librador.dto.patch.Patch;
import nl.tudelft.tam.dto.patch.embeddables.ExtraWorkDefaultPatchDTO;
import nl.tudelft.tam.dto.patch.embeddables.JobOfferDefaultPatchDTO;
import nl.tudelft.tam.model.CoordinatorDefault;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class CoordinatorDefaultPatchDTO extends Patch<CoordinatorDefault> {

	@Embedded
	@Builder.Default
	private JobOfferDefaultPatchDTO jobOfferDefaultPatchDTO = new JobOfferDefaultPatchDTO();

	@Embedded
	@Builder.Default
	private ExtraWorkDefaultPatchDTO extraWorkDefaultPatchDTO = new ExtraWorkDefaultPatchDTO();

	private String coordinatorContractName;

	private String coordinatorContractEmail;

	@Builder.Default
	private Boolean teachersHaveAccess = false;

	public CoordinatorDefaultPatchDTO(JobOfferDefaultPatchDTO jobOfferDefaultPatchDTO,
			ExtraWorkDefaultPatchDTO extraWorkDefaultPatchDTO, String coordinatorContractName,
			String coordinatorContractEmail, Boolean teachersHaveAccess) {
		this.jobOfferDefaultPatchDTO = jobOfferDefaultPatchDTO;
		this.extraWorkDefaultPatchDTO = extraWorkDefaultPatchDTO;
		this.coordinatorContractName = coordinatorContractName;
		this.coordinatorContractEmail = coordinatorContractEmail;
		this.teachersHaveAccess = teachersHaveAccess;
	}

	@Override
	protected void applyOneToOne() {
		data.setJobOfferDefault(jobOfferDefaultPatchDTO.apply(data.getJobOfferDefault(), mapper));
		data.setExtraWorkDefault(extraWorkDefaultPatchDTO.apply(data.getExtraWorkDefault(), mapper));

		data.setCoordinatorContractName(coordinatorContractName);
		data.setCoordinatorContractEmail(coordinatorContractEmail);
	}

	@Override
	protected void validate() {
	}

	private ModelMapper mapper;

	@Override
	protected CoordinatorDefault applyImpl(CoordinatorDefault data, ModelMapper mapper) {
		this.mapper = mapper;
		return super.applyImpl(data, mapper);
	}

}
