/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.patch.embeddables;

import java.time.LocalDate;

import org.springframework.format.annotation.DateTimeFormat;

import jakarta.persistence.Embeddable;
import lombok.*;
import nl.tudelft.librador.dto.patch.Patch;
import nl.tudelft.tam.model.embeddables.JobOfferDefault;

@Embeddable
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class JobOfferDefaultPatchDTO extends Patch<JobOfferDefault> {

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate applicationDeadline;

	private String contractName;

	private String contractBaanCode;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate contractStartDate;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate contractEndDate;

	private String description;

	private String hiringMessage;

	private String rejectMessage;

	@Override
	protected void applyOneToOne() {
		updateNonNull(contractName, data::setContractName);
		updateNonNull(contractBaanCode, data::setContractBaanCode);
		updateNonNull(description, data::setDescription);
		updateNonNull(hiringMessage, data::setHiringMessage);
		updateNonNull(rejectMessage, data::setRejectMessage);

		data.setApplicationDeadline(applicationDeadline);

		data.setContractStartDate(contractStartDate);
		data.setContractEndDate(contractEndDate);
	}

	@Override
	protected void validate() {
	}
}
