/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.view.summary;

import java.time.LocalDate;

import com.vladsch.flexmark.html.HtmlRenderer;
import com.vladsch.flexmark.parser.Parser;
import com.vladsch.flexmark.util.ast.Node;
import com.vladsch.flexmark.util.data.MutableDataSet;

import lombok.*;
import lombok.experimental.SuperBuilder;
import nl.tudelft.librador.dto.view.View;
import nl.tudelft.tam.enums.OfferType;
import nl.tudelft.tam.model.JobOffer;

@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class JobOfferSummaryDTO extends View<JobOffer> {

	private static final MutableDataSet options = new MutableDataSet();
	private static final Parser parser = Parser.builder(options).build();
	private static final HtmlRenderer renderer = HtmlRenderer.builder(options).build();

	private Long id;

	private Long editionId;

	private String name;

	private String contractName;

	private LocalDate contractStartDate;

	private LocalDate contractEndDate;

	private String baanCode;

	private Integer maxHours;

	private LocalDate deadline;

	private String description;

	private String descriptionHtml;

	private String hiringMessage;

	private String hiringMessageHtml;

	private String rejectMessage;

	private String rejectMessageHtml;

	private Boolean requireApplicationText;

	private Boolean draft;

	private OfferType offerType;

	@Override
	public void postApply() {
		super.postApply();

		if (description != null) {
			Node descriptionNode = parser.parse(this.description);
			this.descriptionHtml = renderer.render(descriptionNode);
		}
		if (hiringMessage != null) {
			Node hiringMessageNode = parser.parse(this.hiringMessage);
			this.hiringMessageHtml = renderer.render(hiringMessageNode);
		}
		if (rejectMessage != null) {
			Node rejectMessageNode = parser.parse(this.rejectMessage);
			this.rejectMessageHtml = renderer.render(rejectMessageNode);
		}
	}
}
