/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.model;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Nullable;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Size;
import lombok.*;
import nl.tudelft.tam.enums.Status;

@Data
@Entity
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class Application {

	@Data
	@Embeddable
	@NoArgsConstructor
	@AllArgsConstructor
	public static class AppId implements Serializable {
		private Long personId;
		private Long jobOfferId;
	}

	@NotNull
	@EmbeddedId
	private AppId id;

	@MapsId("jobOfferId")
	@ManyToOne
	private JobOffer jobOffer;

	@NotNull
	@Builder.Default
	private Status status = Status.SUBMITTED;

	@ManyToOne
	private Profile handler;

	@Lob
	@Nullable
	@Size(max = 8192)
	@Column(columnDefinition = "LONGTEXT")
	private String content;

	@NotNull
	@Builder.Default
	@ToString.Exclude
	@EqualsAndHashCode.Exclude
	@OneToMany(mappedBy = "application")
	private List<ApplicationChangeEvent> events = new ArrayList<>();

	@NotNull
	@Builder.Default
	@Column(columnDefinition = "DATE")
	private LocalDateTime createdDate = LocalDateTime.now();

	@NotNull
	@Builder.Default
	@Column(columnDefinition = "DATE")
	private LocalDateTime lastModifiedDate = LocalDateTime.now();

	@PreUpdate
	public void setLastModifiedDate() {
		this.lastModifiedDate = LocalDateTime.now();
	}

	@PrePersist
	public void setCreatedDate() {
		this.createdDate = LocalDateTime.now();
	}

}
