/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.model.records;

import java.time.format.DateTimeFormatter;

import nl.tudelft.labracore.api.dto.EditionDetailsDTO;
import nl.tudelft.labracore.api.dto.PersonSummaryDTO;
import nl.tudelft.tam.model.*;

public record FlexDelftExport(PersonSummaryDTO applicant, Profile applicantProfile, JobOffer jobOffer,
		ExtraWork extraWork, EditionDetailsDTO edition, CoordinatorDefault coordinatorDefault, Double hours,
		PersonSummaryDTO approver, String programName, String batch, String remarks) {

	public FlexDelftExport(PersonSummaryDTO applicant, Profile applicantProfile, ExtraWork extraWork,
			EditionDetailsDTO edition, CoordinatorDefault coordinatorDefault, Double hours,
			String programName, String batch, String remarks) {
		this(applicant, applicantProfile, null, extraWork,
				edition, coordinatorDefault, hours, null, programName, batch, remarks);
	}

	public FlexDelftExport(PersonSummaryDTO applicant, Profile applicantProfile, JobOffer jobOffer,
			EditionDetailsDTO edition, CoordinatorDefault coordinatorDefault,
			PersonSummaryDTO approver, String programName, String batch, String remarks) {
		this(applicant, applicantProfile, jobOffer, null,
				edition, coordinatorDefault, null, approver, programName, batch, remarks);
	}

	private static final DateTimeFormatter exportDateFormatter = DateTimeFormatter.ofPattern("dd-MM-yyyy");

	/**
	 * Generates the export array
	 *
	 * @return the array of strings to export
	 */
	public String[] toExport() {
		if (jobOffer != null) {
			return new String[] {
					applicant.getDisplayName(),
					applicant.getUsername(),
					applicant.getEmail(),
					jobOffer.getContractName(),
					jobOffer.getBaanCode(),
					approver.getDisplayName(),
					approver.getEmail(),
					coordinatorDefault.getCoordinatorContractName(),
					String.valueOf(jobOffer.getMaxHours()),
					applicantProfile.getPayScale().toString(),
					jobOffer.getContractStartDate().format(exportDateFormatter),
					jobOffer.getContractEndDate().format(exportDateFormatter),
					programName,
					batch,
					remarks
			};
		} else {
			return new String[] {
					applicant.getDisplayName(),
					applicant.getEmail(),
					edition.getCourse().getCode(),
					extraWork.getContractName(),
					extraWork.getBaanCode(),
					coordinatorDefault.getCoordinatorContractName(),
					coordinatorDefault.getCoordinatorContractEmail(),
					coordinatorDefault.getCoordinatorContractName(),
					String.valueOf(hours),
					applicantProfile.getPayScale().toString(),
					programName,
					batch,
					remarks
			};
		}
	}
}
