/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import java.time.LocalDate;
import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;

import nl.tudelft.tam.model.ExtraWork;

public interface ExtraWorkRepository extends JpaRepository<ExtraWork, Long> {

	/**
	 * Finds the extraWork by id or throws a ResourceNotFoundException.
	 *
	 * @param  id the id of the extraWork
	 * @return    the ExtraWork object
	 */
	default ExtraWork findByIdOrThrow(Long id) {
		// Spring will actually wrap this in a JpaObjectRetrievalFailureException
		return findById(id)
				.orElseThrow(() -> new ResourceNotFoundException("ExtraWork was not found: " + id));
	}

	List<ExtraWork> findAllByDeadlineIsNullOrDeadlineIsAfter(LocalDate date);

	List<ExtraWork> findAllByDeadlineIsBefore(LocalDate date);

	/**
	 * Finds all extra work that have a deadline in the future.
	 *
	 * @return List of extra work
	 */
	default List<ExtraWork> findAllOpen() {
		return findAllByDeadlineIsNullOrDeadlineIsAfter(LocalDate.now().minusDays(1));
	}

	/**
	 * Finds all extra work that have a deadline in the past.
	 *
	 * @return List of extra work
	 */
	default List<ExtraWork> findAllClosed() {
		return findAllByDeadlineIsBefore(LocalDate.now());
	}

	List<ExtraWork> findAllByEditionIdIsIn(List<Long> editionIds);

	default List<ExtraWork> findAllByEditions(List<Long> editionIds) {
		return findAllByEditionIdIsIn(editionIds);
	}

}
