/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import java.util.List;
import java.util.Optional;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;

import nl.tudelft.tam.enums.Status;
import nl.tudelft.tam.model.RaiseRequest;

public interface RaiseRequestRepository extends JpaRepository<RaiseRequest, Long> {

	/**
	 * Finds the raiseRequest by id or throws a ResourceNotFoundException.
	 *
	 * @param  id the id of the raiseRequest
	 * @return    the raiseRequest object
	 */
	default RaiseRequest findByIdOrThrow(Long id) {
		// Spring will actually wrap this in a JpaObjectRetrievalFailureException
		return findById(id)
				.orElseThrow(() -> new ResourceNotFoundException("RaiseRequest was not found: " + id));
	}

	Optional<RaiseRequest> findByRequestedBy_IdAndStatus(Long id, Status status);

	default Optional<RaiseRequest> findByRequesterAndSubmitted(Long id) {
		return findByRequestedBy_IdAndStatus(id, Status.SUBMITTED);
	}

	List<RaiseRequest> findAllByStatus(Status status);

}
