/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;

import nl.tudelft.tam.model.TrainingType;

public interface TrainingTypeRepository extends JpaRepository<TrainingType, Long> {

	/**
	 * Finds the TrainingType by id or throws a ResourceNotFoundException.
	 *
	 * @param  id the id of the TrainingType
	 * @return    the TrainingType object
	 */
	default TrainingType findByIdOrThrow(Long id) {
		// Spring will actually wrap this in a JpaObjectRetrievalFailureException
		return findById(id)
				.orElseThrow(
						() -> new ResourceNotFoundException("TrainingType was not found: " + id));
	}

	/**
	 * Finds all associated training types by name whilst ignoring the case.
	 *
	 * @param  name      The name of the training type
	 * @param  programId The ID of the program
	 * @return           A list of all training types that correspond to the specified constraints.
	 */
	List<TrainingType> findByNameIgnoreCaseAndProgramId(String name, Long programId);

	/**
	 * Finds all TrainingTypes by program id.
	 *
	 * @param  id The program id to find the training types by
	 * @return    the list of corresponding training types
	 */
	List<TrainingType> findByProgramId(Long id);

}
