/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.security;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import io.sentry.Sentry;
import io.sentry.protocol.User;
import nl.tudelft.labracore.api.ProgramControllerApi;
import nl.tudelft.labracore.api.dto.ProgramSummaryDTO;
import nl.tudelft.labracore.lib.security.LabradorUserHandler;
import nl.tudelft.labracore.lib.security.user.DefaultRole;
import nl.tudelft.labracore.lib.security.user.Person;
import nl.tudelft.tam.enums.PayScale;
import nl.tudelft.tam.model.Profile;
import nl.tudelft.tam.repository.ProfileRepository;

/**
 * Interface for handling user logins on the client implementation.
 *
 * For more information, consult LabraDoor implementation and documentation at
 * https://gitlab.ewi.tudelft.nl/eip/labrador/labradoor/-/wikis/Introduction.
 */

@Service
public class LoginUserHandler implements LabradorUserHandler {

	@Autowired
	private ProfileRepository profileRepository;
	@Autowired
	private ProgramControllerApi programApi;

	/**
	 * Makes changed to the TAM database for the person in question. The method checks whether the person who
	 * just logged in already has a TAM profile. If not, a new one is instantiated for them.
	 *
	 * @param person who has just logged in
	 */
	@Override
	public void handleUserLogin(Person person) {
		Sentry.configureScope(scope -> {
			User user = new User();
			user.setUsername(person.getUsername());
			scope.setTag("DefaultRole", person.getDefaultRole().toString());
		});

		Profile profile = profileRepository.findById(person.getId())
				.orElseGet(() -> profileRepository.save(Profile.builder().id(person.getId()).build()));
		if (person.getDefaultRole() == DefaultRole.STUDENT
				&& Boolean.TRUE.equals(programApi.getAllRelevantPrograms(person.getId())
						.any(p -> p.getType() == ProgramSummaryDTO.TypeEnum.MASTER).block())) {
			profile.setPayScale(PayScale.SA4);
			profileRepository.save(profile);
		}
	}

}
