// noinspection JSUnusedGlobalSymbols

/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

document.addEventListener("DOMContentLoaded", function () {
    $(".overlay[data-closable]").click(function (event) {
        if (event.target !== event.currentTarget) {
            return;
        } else {
            event.preventDefault();
        }
        toggleOverlay($(this).attr("id"));
    });

    $(".m-table").each((tableIndex, tableElement) => {
        const headers = $(tableElement).find(".table_header > th");
        $(tableElement)
            .find("tr")
            .not(".table_header")
            .each((rowIndex, rowElement) => {
                $(rowElement)
                    .find("td")
                    .each((tdIndex, tdElement) => {
                        if (!$(tdElement).attr("data-empty")) {
                            $(tdElement).attr("data-before", $(headers[tdIndex]).text());
                        }
                    });
            });
    });
    // Scroll back to this place after submitting form.
    $("[data-scroll-to='true']").submit(function () {
        const scroll = window.scrollY;
        localStorage.setItem("scrollpos", scroll);
        return true;
    });

    // Scroll to a specific position if this is stored in localStorage.
    const scrollPos = localStorage.getItem("scrollpos");
    if (scrollPos) window.scrollTo(0, scrollPos);
    localStorage.removeItem("scrollpos");
});

let overlayStack = [];
function toggleOverlay(id) {
    if (overlayStack.length > 0 && overlayStack[overlayStack.length - 1] !== id) {
        document.getElementById(overlayStack[overlayStack.length - 1]).classList.add("hidden");
    }
    let ovl = document.getElementById(id);
    ovl.classList.toggle("hidden");
    if (ovl.classList.contains("hidden")) {
        overlayStack.pop();
        if (overlayStack.length > 0) {
            document
                .getElementById(overlayStack[overlayStack.length - 1])
                .classList.remove("hidden");
        }
    } else {
        overlayStack.push(id);
    }
}

function toggleDatePicker(id) {
    const datepicker = document.getElementById(id);
    datepicker.classList.toggle("hidden");
    datepicker.value = null;
    if (datepicker.hasAttribute("required")) {
        datepicker.removeAttribute("required");
    } else {
        datepicker.setAttribute("required", "required");
    }
}

function toggleInput(id) {
    const input = document.getElementById(id);
    input.classList.toggle("invisible");

    if (input.hasAttribute("required")) {
        input.removeAttribute("required");
    } else {
        input.setAttribute("required", "required");
    }
}

function forceCheckbox(id) {
    const checkbox = document.getElementById(id);
    checkbox.checked = true;
    checkbox.disabled = true;
}

function toggleDatePickerWith(id, valid) {
    const ovl = document.getElementById(id);
    ovl.classList.toggle("hidden");
    if (ovl.classList.contains("hidden")) {
        ovl.value = null;
    } else {
        ovl.value = document.getElementById(valid).value;
    }
}

function newDeclarationOverlay(workId) {
    document.getElementById("new-declaration-overlay").showModal();
    document.getElementById("new-declaration-time-error").classList.add("hidden");
    if (workId !== null) {
        document.getElementById("new-declaration-id").value = workId;
        document.getElementById("new-declaration-name").value = document.getElementById(
            "work-name-" + workId
        ).innerText;
        document.getElementById("new-declaration-remaining-time").value = document.getElementById(
            "work-remaining-time-" + workId
        ).innerText;
    } else {
        document.getElementById("new-declaration-id").value = null;
        document.getElementById("new-declaration-name").value = "";
        document.getElementById("new-declaration-remaining-time").value = "";
    }
}

let timeout;
function newDeclarationUpdateTime() {
    clearTimeout(timeout);
    timeout = setTimeout(function () {
        const timeEl = document.getElementById("new-declaration-time");
        const amountEl = document.getElementById("new-declaration-amount");
        const hourIndex = timeEl.value.indexOf(":");
        let m = 0;
        let h = 0;
        if (hourIndex !== -1) {
            h = parseInt(timeEl.value.substr(0, hourIndex), 10);
            if (isNaN(h)) h = 0;
            m = Math.round(parseInt(timeEl.value.substr(hourIndex + 1, 2), 10) / 15);
            if (isNaN(m)) m = 0;
        } else {
            h = parseInt(timeEl.value.substr(0, 2), 10);
            if (isNaN(h)) h = 0;
        }
        if (m > 3) {
            h += 1;
            m = 0;
        }
        amountEl.value = h * 4 + m;
        let timeStr = "";
        if (h < 10) timeStr += "0" + h;
        else timeStr += h;
        timeStr += ":";
        if (m === 0) timeStr += "00";
        else timeStr += m * 15;
        timeEl.value = timeStr;
        if (timeEl.value === "00:00") {
            document.getElementById("new-declaration-time-error").classList.remove("hidden");
        } else {
            document.getElementById("new-declaration-time-error").classList.add("hidden");
        }
    }, 1000);
}

function newOfferOverlay(type, editionId) {
    if (editionId !== null) {
        document.getElementById("new-" + type + "-edition-id").value = editionId;
        document.getElementById("new-" + type + "-edition-name").value = document.getElementById(
            "edition-" + editionId
        ).innerText;
        document.getElementById("new-" + type + "-course-name").value = document.getElementById(
            "course-" + editionId
        ).innerText;
        if (type === "extra-work") {
            if (
                document.getElementById("default-contract-name-extra-work-" + editionId).value !==
                ""
            ) {
                document.getElementById("new-extra-work-contract-name").value =
                    document.getElementById("default-contract-name-extra-work-" + editionId).value;
            }
            if (document.getElementById("default-baanCode-extra-work-" + editionId).value !== "") {
                document.getElementById("new-extra-work-baan-code").value = document.getElementById(
                    "default-baanCode-extra-work-" + editionId
                ).value;
            } else {
                // No Default Baan-Code
                toggleInput("new-extra-work-baan-code");
                forceCheckbox("new-extra-work-baan-code-bool");
            }
            if (
                document.getElementById("default-declaration-deadline-extra-work-" + editionId)
                    .value !== ""
            ) {
                toggleDatePickerWith(
                    "new-extra-work-deadline",
                    "default-declaration-deadline-extra-work-" + editionId
                );
                document.getElementById("new-extra-work-deadline-bool").checked = true;
            }
        } else if (type === "job-offer") {
            if (
                document.getElementById("default-application-deadline-job-offer-" + editionId)
                    .value !== ""
            ) {
                toggleDatePickerWith(
                    "new-job-offer-deadline",
                    "default-application-deadline-job-offer-" + editionId
                );
                document.getElementById("new-job-offer-deadline-bool").checked = true;
            }
            if (
                document.getElementById("default-contract-name-job-offer-" + editionId).value !== ""
            ) {
                document.getElementById("new-job-offer-contract-name").value =
                    document.getElementById("default-contract-name-job-offer-" + editionId).value;
            }
            if (document.getElementById("default-baanCode-job-offer-" + editionId).value !== "") {
                document.getElementById("new-job-offer-baan-code").value = document.getElementById(
                    "default-baanCode-job-offer-" + editionId
                ).value;
            } else {
                // No Default Baan-Code
                toggleInput("new-job-offer-baan-code");
                forceCheckbox("new-job-offer-baan-code-bool");
            }
            if (
                document.getElementById("default-contract-startDate-job-offer-" + editionId)
                    .value !== ""
            ) {
                document.getElementById("new-job-offer-contract-start").value =
                    document.getElementById(
                        "default-contract-startDate-job-offer-" + editionId
                    ).value;
            }
            if (
                document.getElementById("default-contract-endDate-job-offer-" + editionId).value !==
                ""
            ) {
                document.getElementById("new-job-offer-contract-end").value =
                    document.getElementById(
                        "default-contract-endDate-job-offer-" + editionId
                    ).value;
            }
            if (
                document.getElementById("default-description-job-offer-" + editionId).value !== ""
            ) {
                document.getElementById("new-job-offer-description").value =
                    document.getElementById("default-description-job-offer-" + editionId).value;
            }
            if (
                document.getElementById("default-hiring-message-job-offer-" + editionId).value !==
                ""
            ) {
                document.getElementById("new-job-offer-hiring-message").value =
                    document.getElementById("default-hiring-message-job-offer-" + editionId).value;
            }
            if (
                document.getElementById("default-reject-message-job-offer-" + editionId).value !==
                ""
            ) {
                document.getElementById("new-job-offer-reject-message").value =
                    document.getElementById("default-reject-message-job-offer-" + editionId).value;
            }
        }
    } else {
        document.getElementById("new-" + type + "-edition-id").value = null;
        document.getElementById("new-" + type + "-edition-name").value = null;
        document.getElementById("new-" + type + "-course-name").value = null;
    }
}

/**
 * Toggles an accordion on click
 * @param id The id of the row to toggle
 */
function accordionToggle(id) {
    if (document.getElementById(id).classList.contains("collapsed")) {
        // Currently Hidden
        document.getElementById(id).classList.remove("collapsed");
    } else {
        // Currently Visible
        document.getElementById(id).classList.add("collapsed");
    }
}

/**
 * Stop the propagation of the click event.
 *
 * @param event The event for which to stop the propagation
 * @param id The id of the overlay to close
 */
function outerClose(event, id) {
    if (event.target !== event.currentTarget) {
        return;
    }
    toggleOverlay(id);
}

function isDurationGreaterThanRemainder(duration) {
    const hourIndex = duration.indexOf(":");
    const timeRemainingIndex =
        document.getElementById("new-declaration-remaining-time").value.indexOf(":") + 1;

    let numberDuration =
        +duration.substr(0, hourIndex) + duration.substr(hourIndex + 1, duration.length) / 60;
    let numberRemaining = document
        .getElementById("new-declaration-remaining-time")
        .value.substr(timeRemainingIndex);

    return numberDuration > numberRemaining;
}

function declarationSubmit() {
    const timeEl = document.getElementById("new-declaration-time");
    const reasonEl = document.getElementById("new-declaration-reason");
    if (timeEl.value === "00:00") {
        document.getElementById("new-declaration-time-error").classList.remove("hidden");
        timeEl.focus();
    } else if (isDurationGreaterThanRemainder(timeEl.value)) {
        document.getElementById("new-declaration-time-remaining-error").classList.remove("hidden");
        timeEl.focus();
    } else if (!reasonEl.value || reasonEl.value.length < 10 || reasonEl.value.length > 140) {
        document.getElementById("new-declaration-reason-error").classList.remove("hidden");
        reasonEl.focus();
    } else {
        document.getElementById("new-declaration-time-error").classList.add("hidden");
        document.getElementById("new-declaration-time-remaining-error").classList.add("hidden");
        document.getElementById("new-declaration-reason-error").classList.add("hidden");
        document.getElementById("new-declaration-form").submit();
    }
}

function showHideEmailNotificationFrequency(emailNotificationsEnabledInput) {
    const emailNotificationFrequencyInput = document.getElementById("emailNotificationFrequency");
    const emailNotificationFrequencyLabel = document.getElementById(
        "labelForEmailNotificationFrequency"
    );

    if (emailNotificationsEnabledInput.checked) {
        emailNotificationFrequencyInput.removeAttribute("hidden");
        emailNotificationFrequencyLabel.removeAttribute("hidden");
    } else {
        emailNotificationFrequencyInput.setAttribute("hidden", "");
        emailNotificationFrequencyLabel.setAttribute("hidden", "");
    }
}
