/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.model;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.labracore.api.dto.CohortSummaryDTO;
import nl.tudelft.labracore.api.dto.CourseDetailsDTO;
import nl.tudelft.labracore.api.dto.CourseSummaryDTO;
import nl.tudelft.labracore.api.dto.EditionDetailsDTO;
import nl.tudelft.tam.security.AuthorisationService;
import nl.tudelft.tam.service.CourseService;

@AutoConfigureMockMvc
@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class CourseControllerTest {

	@Autowired
	MockMvc mvc;

	@MockBean
	AuthorisationService authService;

	@MockBean
	CourseService courseService;

	EditionDetailsDTO edition;

	CourseSummaryDTO courseSummary;

	CourseDetailsDTO courseDetails;

	final long EDITION_ID = 7000;
	final long COURSE_ID = 2222;

	@BeforeEach
	void setUp() {
		courseSummary = new CourseSummaryDTO().name("course-name").code("course-code").id(COURSE_ID);
		courseDetails = new CourseDetailsDTO().name("course-name").code("course-code").id(COURSE_ID);
		edition = new EditionDetailsDTO().id(EDITION_ID)
				.course(courseSummary)
				.cohort(new CohortSummaryDTO().name("cohort-name")).name("edition-name")
				.isArchived(false);
	}

	@Test
    @WithUserDetails("student")
    void getCourseTest() throws Exception {
		when(authService.canManageCourse(anyLong())).thenReturn(true);
        when(courseService.getCourseById(anyLong())).thenReturn(courseDetails);
        mvc.perform(get("/course/{id}", COURSE_ID))
                .andExpect(model().attribute("course", courseDetails))
                .andExpect(status().isOk());
        verify(courseService).getCourseById(COURSE_ID);
    }

}
