/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.model;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import application.test.TestTAMApplication;
import jakarta.persistence.Entity;
import nl.tudelft.tam.repository.JobHolderRepository;
import nl.tudelft.tam.repository.ProjectIdRepository;
import nl.tudelft.tam.repository.ProjectRepository;
import nl.tudelft.tam.repository.TAMProgramRepository;

@SpringBootTest(classes = { TestTAMApplication.class, JobHolderTest.class })
public class JobHolderTest {

	private final ProjectRepository projectRepository;
	private final TAMProgramRepository programRepository;
	private final ProjectIdRepository projectIdRepository;
	private final EditionRepository editionRepository;
	private final JobHolderRepository jobHolderRepository;

	@Autowired
	public JobHolderTest(ProjectRepository projectRepository, TAMProgramRepository programRepository,
			ProjectIdRepository projectIdRepository, EditionRepository editionRepository,
			JobHolderRepository jobHolderRepository) {
		this.projectRepository = projectRepository;
		this.programRepository = programRepository;
		this.projectIdRepository = projectIdRepository;
		this.editionRepository = editionRepository;
		this.jobHolderRepository = jobHolderRepository;
	}

	@Test
	public void jobHolderIdsCanBeAutoGeneratedOrSetManually() {
		TAMProgram program = programRepository.save(TAMProgram.builder().id(1L).build());

		ProjectId projectId = projectIdRepository.save(new ProjectId());
		Project project = new Project(projectId, "name", program);
		project = projectRepository.save(project);
		assertThat(project.getId()).isEqualTo("project-" + projectId.getId());

		Edition edition = new Edition("edition-1");
		edition = editionRepository.save(edition);
		assertThat(edition.getId()).isEqualTo("edition-1");

		assertThat(jobHolderRepository.existsById("project-" + projectId.getId())).isTrue();
		assertThat(jobHolderRepository.existsById("edition-1")).isTrue();
	}

	// TODO remove in favour of TAMEdition when it exists
	@Entity
	public static class Edition extends JobHolder {
		public Edition() {
		}

		public Edition(String id) {
			super(id);
		}
	}

}
