/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.time.LocalDate;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.tam.model.CoordinatorDefault;
import nl.tudelft.tam.model.embeddables.ExtraWorkDefault;
import nl.tudelft.tam.model.embeddables.JobOfferDefault;

@SpringBootTest(classes = TestTAMApplication.class)
public class CoordinatorDefaultRepositoryTest {

	@Autowired
	private CoordinatorDefaultRepository repository;

	@BeforeEach
	void setup() {
		repository.deleteAll();
	}

	@Test
	@Transactional
	void findByIdOrThrow() {
		assertThrows(ResourceNotFoundException.class, () -> repository.findByIdOrThrow(1L));

		JobOfferDefault joDefault = JobOfferDefault.builder().contractName("TEST JO")
				.contractBaanCode("TEST BAAN JO").contractStartDate(LocalDate.now())
				.contractEndDate(LocalDate.now().plusWeeks(1)).applicationDeadline(LocalDate.now()).build();
		ExtraWorkDefault ewDefault = ExtraWorkDefault.builder().contractName("TEST EW")
				.contractBaanCode("TEST BAAN EW").build();

		CoordinatorDefault coordDefault = repository.save(CoordinatorDefault.builder().programId(1L)
				.jobOfferDefault(joDefault).extraWorkDefault(ewDefault).build());

		assertThat(repository.findByIdOrThrow(1L)).isEqualTo(coordDefault);
	}
}
