/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.tam.enums.PayScale;
import nl.tudelft.tam.enums.Status;
import nl.tudelft.tam.model.Profile;
import nl.tudelft.tam.model.RaiseRequest;
import nl.tudelft.tam.test.TestUserDetailsService;

@SpringBootTest(classes = TestTAMApplication.class)
class RaiseRequestRepositoryTest {

	@Autowired
	RaiseRequestRepository raiseRequestRepository;

	@Autowired
	ProfileRepository profileRepository;

	Profile profile;

	@BeforeEach
	void setup() {
		profile = profileRepository.save(Profile.builder()
				.id(TestUserDetailsService.id)
				.build());
	}

	@Test
	@Transactional
	void findByIdOrThrow() {
		Long id = 6897301578L;
		PayScale payScale = PayScale.SA3;
		String proof = "proof-6846518";

		assertThrows(ResourceNotFoundException.class, () -> raiseRequestRepository.findByIdOrThrow(id));

		RaiseRequest request = raiseRequestRepository.save(RaiseRequest.builder()
				.id(id)
				.requestedBy(profile)
				.payScale(payScale)
				.build());

		assertThat(raiseRequestRepository.findByIdOrThrow(request.getId()))
				.isEqualTo(request);
	}

	@Test
	@Transactional
	void findByRequesterAndSubmitted() {

		assertThat(raiseRequestRepository.findByRequesterAndSubmitted(profile.getId())).isEmpty();

		RaiseRequest request2 = raiseRequestRepository.save(RaiseRequest.builder()
				.requestedBy(profile)
				.payScale(PayScale.SA2)
				.status(Status.REJECTED_BY_STUDENT)
				.build());

		RaiseRequest request3 = raiseRequestRepository.save(RaiseRequest.builder()
				.requestedBy(profile)
				.payScale(PayScale.SA3)
				.status(Status.REJECTED_BY_TEACHER)
				.build());

		RaiseRequest request4 = raiseRequestRepository.save(RaiseRequest.builder()
				.requestedBy(profile)
				.payScale(PayScale.SA4)
				.status(Status.SUBMITTED)
				.build());

		Optional<RaiseRequest> found = raiseRequestRepository.findByRequesterAndSubmitted(profile.getId());

		assertThat(found).isNotEmpty();
		assertThat(found.get()).isEqualTo(request4);
	}

	@Test
	@Transactional
	void findAllByStatus() {
		for (Status s : Status.values()) {
			assertThat(raiseRequestRepository.findAllByStatus(s)).isEmpty();
		}

		RaiseRequest request1 = raiseRequestRepository.save(RaiseRequest.builder()
				.requestedBy(profile)
				.payScale(PayScale.SA1)
				.status(Status.ACCEPTED)
				.build());

		RaiseRequest request2 = raiseRequestRepository.save(RaiseRequest.builder()
				.requestedBy(profile)
				.payScale(PayScale.SA2)
				.status(Status.ACCEPTED)
				.build());

		List<RaiseRequest> list = raiseRequestRepository.findAllByStatus(Status.ACCEPTED);
		assertThat(list.size()).isEqualTo(2);
		assertThat(list.contains(request1)).isTrue();
		assertThat(list.contains(request2)).isTrue();
	}
}
