/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.labracore.api.CohortControllerApi;
import nl.tudelft.labracore.api.dto.CohortDetailsDTO;
import nl.tudelft.labracore.api.dto.CohortSummaryDTO;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class CohortServiceTest {

	private final Long COHORT_ID = 155L;

	@Autowired
	public CohortService cohortService;

	@MockBean
	public CohortControllerApi cohortControllerApi;

	@Test
	public void getAllCohorts() {
		CohortSummaryDTO cohort = new CohortSummaryDTO().id(COHORT_ID);

		when(cohortControllerApi.getAllCohorts()).thenReturn(Flux.just(cohort));

		assertThat(cohortService.getAllCohorts()).isEqualTo(List.of(cohort));

		verify(cohortControllerApi).getAllCohorts();
	}

	@Test
	public void getOrThrowById() {
		CohortDetailsDTO cohort = new CohortDetailsDTO().id(COHORT_ID);

		when(cohortControllerApi.getCohortById(anyLong())).thenReturn(Mono.just(cohort));

		assertThat(cohortService.getOrThrowById(COHORT_ID)).isEqualTo(cohort);
	}

	@Test
	public void getCohortsById() {
		CohortDetailsDTO cohort = new CohortDetailsDTO().id(COHORT_ID);

		when(cohortControllerApi.getAllCohortsById(any())).thenReturn(Flux.just(cohort));

		assertThat(cohortService.getCohortsById(List.of(COHORT_ID))).isEqualTo(List.of(cohort));
	}

	@Test
	public void get() {
		CohortDetailsDTO cohort = new CohortDetailsDTO().id(COHORT_ID);

		when(cohortControllerApi.getCohortById(any())).thenReturn(Mono.just(cohort));

		assertThat(cohortService.get(List.of(COHORT_ID, COHORT_ID))).isEqualTo(List.of(cohort));
	}

}
