/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.anyList;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Set;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.labracore.api.PersonControllerApi;
import nl.tudelft.labracore.api.dto.PersonSearchDTO;
import nl.tudelft.labracore.api.dto.PersonSummaryDTO;
import nl.tudelft.tam.model.Coordinator;
import nl.tudelft.tam.repository.CoordinatorRepository;
import reactor.core.publisher.Mono;

@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class CoordinatorServiceTest {

	Coordinator coordinator;

	Long COORDINATOR_ID;

	@Autowired
	CoordinatorService service;

	@Autowired
	CoordinatorRepository repository;

	@Autowired
	PersonControllerApi personApi;

	@BeforeEach
	void setup() {
		Coordinator tempCoord = new Coordinator();
		tempCoord.setProgramId(1L);
		tempCoord.setPersonId(1L);
		coordinator = repository.save(tempCoord);
		COORDINATOR_ID = coordinator.getPersonId();
	}

	@Test
	void findAllByPersonIdTest() {
		assertThat(service.findAllByPersonId(COORDINATOR_ID)).isEqualTo(List.of(coordinator));
	}

	@Test
	void existsByPersonIdTest() {
		assertThat(service.existsByPersonId(1L)).isTrue();
		assertThat(service.existsByPersonId(2L)).isFalse();
	}

	@Test
	void existsByPersonIdAndProgramIdTest() {
		assertThat(service.existsByPersonIdAndProgramId(1L, 1L)).isTrue();
		assertThat(service.existsByPersonIdAndProgramId(2L, 2L)).isFalse();
	}

	@Test
	void setCoordinators() {
		when(personApi.searchForPeople(anyList()))
				.thenReturn(Mono.just(new PersonSearchDTO().people(Set.of(new PersonSummaryDTO().id(5L)))));
		service.setCoordinatorsForProgram(1L, List.of("username"));
		List<Coordinator> coordinators = service.getCoordinatorsByProgram(1L);
		assertThat(coordinators).hasSize(1);
		assertThat(coordinators.get(0).getPersonId()).isEqualTo(5L);
	}

}
