/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.service;

import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.labracore.api.dto.PersonSummaryDTO;
import nl.tudelft.tam.dto.create.EmailCreateDTO;

@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class EmailServiceTest {

	private static final String ADDRESS1 = "test1@tudelft.nl";
	private static final String ADDRESS2 = "test2@tudelft.nl";

	private static final EmailCreateDTO MESSAGE_CREATE = EmailCreateDTO.builder()
			.subject("Test subject")
			.template("html/test")
			.build();

	@MockBean
	public EmailService emailService;

	@BeforeEach
	void setUp() {
		MockitoAnnotations.openMocks(this);
	}

	@Test
	public void sendEmailToPeopleSendsToAll() {
		when(emailService.sendEmailToPeople(anyList(), any(EmailCreateDTO.class))).thenCallRealMethod();

		emailService.sendEmailToPeople(
				List.of(new PersonSummaryDTO().email(ADDRESS1), new PersonSummaryDTO().email(ADDRESS2)),
				MESSAGE_CREATE);

		verify(emailService).sendEmail(List.of(ADDRESS1, ADDRESS2), MESSAGE_CREATE);
	}
}
