/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.service;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.tam.dto.patch.NotificationPatchDTO;
import nl.tudelft.tam.dto.patch.ShirtPatchDTO;
import nl.tudelft.tam.enums.NotificationFrequency;
import nl.tudelft.tam.enums.ShirtGender;
import nl.tudelft.tam.enums.ShirtSize;
import nl.tudelft.tam.model.Profile;
import nl.tudelft.tam.repository.ProfileRepository;

@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class ProfileServiceTest {

	@Autowired
	ProfileService service;

	@Autowired
	ProfileRepository repository;

	Profile profile;
	Long PROFILE_ID;

	@BeforeEach
	void setup() {
		profile = repository.save(Profile.builder().id(1L).build());
		PROFILE_ID = profile.getId();
	}

	@Test
	void findByIdOrThrowTest() {
		assertThat(service.findByIdOrThrow(PROFILE_ID)).isEqualTo(profile);
	}

	@Test
	void updatePoloDataTest() {
		ShirtSize size = ShirtSize.M;
		ShirtGender gender = ShirtGender.FEMALE;

		assertThat(profile.getShirtSize()).isNull();
		assertThat(profile.getShirtGender()).isNull();

		service.updatePoloData(PROFILE_ID, new ShirtPatchDTO(size, gender));

		profile = repository.findByIdOrThrow(PROFILE_ID);
		assertThat(profile.getShirtSize()).isEqualTo(size);
		assertThat(profile.getShirtGender()).isEqualTo(gender);
	}

	@Test
	void updateNotificationPreferencesTest() {
		NotificationFrequency frequency = NotificationFrequency.Daily;

		assertThat(profile.getEmailNotificationFrequency()).isEqualTo(NotificationFrequency.Weekly);
		assertThat(profile.getEmailNotifications()).isTrue();

		service.updateNotificationPreferences(PROFILE_ID, new NotificationPatchDTO(false, frequency));

		profile = repository.findByIdOrThrow(PROFILE_ID);
		assertThat(profile.getEmailNotificationFrequency()).isEqualTo(frequency);
		assertThat(profile.getEmailNotifications()).isFalse();
	}
}
