/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.when;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import jakarta.persistence.EntityManager;
import nl.tudelft.labracore.api.ProgramControllerApi;
import nl.tudelft.labracore.api.dto.ProgramDetailsDTO;
import nl.tudelft.tam.repository.TAMProgramRepository;
import reactor.core.publisher.Mono;

@Transactional
@SpringBootTest(classes = TestTAMApplication.class)
public class ProgramServiceTest {

	@Autowired
	ProgramService service;

	@Autowired
	ProgramControllerApi programApi;

	@Autowired
	TAMProgramRepository programRepository;

	@Autowired
	EntityManager entityManager;

	@Test
	void getTest() {
		ProgramDetailsDTO program = new ProgramDetailsDTO().id(1L).name("Test Program");

		when(programApi.getProgramById(anyLong())).thenReturn(Mono.just(program));

		assertThat(service.get(1L)).isEqualTo(program);
	}

	@Test
	void getTAMProgram() {
		service.getTAMProgram(1L);
		assertThat(programRepository.existsById(1L)).isTrue();
	}

	//	@Test
	//	void addAndRemoveAdvertisements() {
	//		service.addAdvertisedToProgram(1L, 2L);
	//		service.addAdvertisedToProgram(1L, 3L);
	//		entityManager.refresh(programRepository.saveAndFlush(service.getTAMProgram(1L)));
	//		entityManager.refresh(programRepository.saveAndFlush(service.getTAMProgram(2L)));
	//		entityManager.refresh(programRepository.saveAndFlush(service.getTAMProgram(3L)));
	//		assertThat(service.getProgramIdsAdvertisingTo(2L)).containsExactlyInAnyOrder(1L);
	//		assertThat(service.getProgramIdsAdvertisingTo(3L)).containsExactlyInAnyOrder(1L);
	//		assertThat(service.getProgramIdsAdvertisedToBy(1L)).containsExactlyInAnyOrder(2L, 3L);
	//
	//		service.removeAdvertisedFromProgram(1L, 2L);
	//		entityManager.refresh(programRepository.saveAndFlush(service.getTAMProgram(1L)));
	//		entityManager.refresh(programRepository.saveAndFlush(service.getTAMProgram(2L)));
	//		entityManager.refresh(programRepository.saveAndFlush(service.getTAMProgram(3L)));
	//		assertThat(service.getProgramIdsAdvertisingTo(2L)).isEmpty();
	//		assertThat(service.getProgramIdsAdvertisingTo(3L)).containsExactlyInAnyOrder(1L);
	//		assertThat(service.getProgramIdsAdvertisedToBy(1L)).containsExactlyInAnyOrder(3L);
	//	}

}
