/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.startup;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.time.LocalDate;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.boot.test.mock.mockito.MockBean;

import nl.tudelft.tam.model.JobOffer;
import nl.tudelft.tam.repository.JobOfferRepository;

public class PublishAllJobOffersTest {

	private PublishAllJobOffersService service;

	@MockBean
	private JobOfferRepository jobOfferRepository;

	JobOffer jobOffer1, jobOffer2;

	@BeforeEach
	void setUp() {
		jobOfferRepository = Mockito.mock(JobOfferRepository.class);
		service = new PublishAllJobOffersService(jobOfferRepository);
		jobOffer1 = JobOffer.builder()
				.id(4889L)
				.editionId(87956L)
				.name("Test-Job-open")
				.deadline(LocalDate.now().plusDays(7))
				.editionId(1L)
				.maxHours(100)
				.draft(false)
				.build();

		jobOffer2 = JobOffer.builder()
				.id(7894165L)
				.editionId(8946513L)
				.name("Test-Job-closed")
				.deadline(LocalDate.now().minusDays(7))
				.maxHours(100)
				.draft(true)
				.build();
	}

	@Test
	public void testPublishAllJobOffers() {
		List<JobOffer> jobOffers = List.of(jobOffer1, jobOffer2);
		when(jobOfferRepository.findAll()).thenReturn(jobOffers);
		assertTrue(jobOffer2.getDraft());

		service.publishAllJobOffers();

		verify(jobOfferRepository).findAll();

		assertFalse(jobOffer1.getDraft());
		assertFalse(jobOffer2.getDraft());
	}
}
